@@ -58,8 +58,12 @@ impl Socket {
         let mut cmsg = cmsg_space!([RawFd; MAX_FDS_OUT]);
         let iov = [uio::IoVec::from_mut_slice(buffer)];
 
-        let msg =
-            socket::recvmsg(self.fd, &iov[..], Some(&mut cmsg), socket::MsgFlags::MSG_DONTWAIT)?;
+        let msg = socket::recvmsg(
+            self.fd,
+            &iov[..],
+            Some(&mut cmsg),
+            socket::MsgFlags::MSG_DONTWAIT | socket::MsgFlags::MSG_CMSG_CLOEXEC,
+        )?;
 
         let mut fd_count = 0;
         let received_fds = msg.cmsgs().flat_map(|cmsg| match cmsg {
@@ -195,7 +199,7 @@ impl BufferedSocket {
             if !self.attempt_write_message(msg)? {
                 // If this fails again, this means the message is too big
                 // to be transmitted at all
-                return Err(::nix::Error::Sys(::nix::errno::Errno::E2BIG));
+                return Err(::nix::Error::E2BIG);
             }
         }
         Ok(())
@@ -222,7 +226,7 @@ impl BufferedSocket {
         };
         if in_bytes == 0 {
             // the other end of the socket was closed
-            return Err(::nix::Error::Sys(::nix::errno::Errno::EPIPE));
+            return Err(::nix::Error::EPIPE);
         }
         // advance the storage
         self.in_data.advance(in_bytes / 4 + if in_bytes % 4 > 0 { 1 } else { 0 });
@@ -351,7 +355,7 @@ impl BufferedSocket {
             // now, try to get more data
             match self.fill_incoming_buffers() {
                 Ok(()) => (),
-                Err(e @ ::nix::Error::Sys(::nix::errno::Errno::EAGAIN)) => {
+                Err(e @ ::nix::Error::EAGAIN) => {
                     // stop looping, returning Ok() or EAGAIN depending on whether messages
                     // were dispatched
                     if dispatched == 0 {
